#ifndef __KVS_VISCLIENT_ARGUMENT_H__
#define __KVS_VISCLIENT_ARGUMENT_H__

#include "kvs/CommandLine"
#include "kvs/TransferFunction"
#include <kvs/Directory>
#include <kvs/File>

namespace kvs
{
namespace visclient
{

class Argument : public kvs::CommandLine
{
public:
    std::string header;
    kvs::TransferFunction transfer_function;
    size_t subpixel_level_abstract;
    size_t subpixel_level_detailed;
    size_t repeat_level_abstract;
    size_t repeat_level_detailed;
    std::string hostname;
    size_t port;
// APPEND START BY)M.Tanaka 2015.03.03
    size_t particle_limit;
    size_t particle_density;
    std::string param_file;
// APPEND START BY)M.Tanaka 2015.03.11
    std::string samplingType;
    std::string detailedparticletransfertype;
    std::string shadingtype;
    bool samplingType_f;
    bool detailedparticletransferType_f;
    bool shadingType_f;
// APPEND END   BY)M.Tanaka 2015.03.11
    std::string viewer;
// APPEND END   BY)M.Tanaka 2015.03.03
    std::string server_subvol_dir;

// APPEND START BY)M.Tanaka 2015.03.05
    bool subpixel_level_detailed_f;
    bool repeat_level_detailed_f;
    bool particle_limit_f;
    bool particle_density_f;
    bool viewer_f;
    bool tffile_f; // APPEND Fj 2015.03.06
    std::string tffile_name;
// APPEND END   BY)M.Tanaka 2015.03.05
    std::string pout_path_name; // APPEND Fj 2015.04.21
    std::string image_base_name;
    std::string pin_dir[10];

    Argument( int argc, char** argv ):
        kvs::CommandLine( argc, argv )
    {
        // Add help option (generate help message automatically).
        add_help_option();
//      add_value( "If \"s\" is specified, it starts in standalone mode.", false );
        add_option( "tf", "transfer_function", 1, false );
// MODIFIED START BY)M.Tanaka 2015.03.03
//		add_option( "sl","subpixel_level",1,false);
//		add_option( "rl","repeat_level",1, false);
// APPEND START BY)M.Tanaka 2015.03.11
        add_option( "S", "sampling type", 1, false );
        add_option( "tdata", "detailed particle transfer type", 1, false );
        add_option( "shading", "shading", 1, false );
// APPEND END   BY)M.Tanaka 2015.03.11
        add_option( "viewer", "resolution wxh", 1, false );
        add_option( "plimit", "particle_limit", 1, false );
        add_option( "pd", "particle_density", 1, false );
        add_option( "hn", "hostname", 1, false );
        add_option( "p", "port", 1, false );
        add_option( "pa", "parameter file", 1, false );
        add_option( "vin", "server subvolume directory", 1, false );
// MODIFIED END   BY)M.Tanaka 2015.03.03
        add_option( "pout", "particle file output directory.", 1, false ); // // APPEND By Fj 2015.04.21
        add_option( "iout", "image file output directory.", 1, false );
        add_option( "pin1", "particle directory No.1", 1, false );
        add_option( "pin2", "particle directory No.2", 1, false );
        add_option( "pin3", "particle directory No.3", 1, false );
        add_option( "pin4", "particle directory No.4", 1, false );
        add_option( "pin5", "particle directory No.5", 1, false );
        add_option( "pin6", "particle directory No.6", 1, false );
        add_option( "pin7", "particle directory No.7", 1, false );
        add_option( "pin8", "particle directory No.8", 1, false );
        add_option( "pin9", "particle directory No.9", 1, false );
        add_option( "pin10", "particle directory No.10", 1, false );


        if ( !this->parse() ) exit( EXIT_FAILURE );

        subpixel_level_abstract = 2;
        repeat_level_abstract = 1;
        subpixel_level_detailed = 2; /* 2 */
        repeat_level_detailed = 1; /* 16 */
        hostname = "localhost";
        port = 60000;
        server_subvol_dir = "";
// APPEND START BY)M.Tanaka 2015.03.11
        samplingType = "u";
        samplingType_f = false;
        detailedparticletransfertype = "all";
        detailedparticletransferType_f = false;
        shadingtype = "";
        shadingType_f = false;
// APPEND END   BY)M.Tanaka 2015.03.11
        subpixel_level_detailed_f = false;
        repeat_level_detailed_f = false;
        particle_limit_f = false;
        particle_density_f = false;
        viewer_f = false;
        tffile_f = false;


        if ( this->hasOption( "tf" ) )
        {
            tffile_f = true;
            tffile_name = this->optionValue<std::string>( "tf" );
        }
        else
        {
            transfer_function.create( 256 );//transfer_function.read( "defalt.kvsml" );
        }

// MODIFIED START BY)M.Tanaka 2015.03.03
        if ( this->hasOption( "sl" ) )
        {
            subpixel_level_detailed = this->optionValue<size_t>( "sl" );
            subpixel_level_detailed_f = true;
        }
        if ( this->hasOption( "rl" ) )
        {
            repeat_level_detailed = this->optionValue<size_t>( "rl" );
            repeat_level_detailed_f = true;
        }
// APPEND START BY)M.Tanaka 2015.03.11
        if ( this->hasOption( "S" ) )
        {
            samplingType = this->optionValue<std::string>( "S" );
            samplingType_f = true;
        }
        if ( this->hasOption( "tdata" ) )
        {
            detailedparticletransfertype = this->optionValue<std::string>( "tdata" );
            detailedparticletransferType_f = true;
        }
        if ( this->hasOption( "shading" ) )
        {
            shadingtype = this->optionValue<std::string>( "shading" );
            shadingType_f = true;
        }
// APPEND END   BY)M.Tanaka 2015.03.11
        if ( this->hasOption( "viewer" ) )
        {
            viewer = this->optionValue<std::string>( "viewer" );
            viewer_f = true;
        }
        if ( this->hasOption( "plimit" ) )
        {
            particle_limit = this->optionValue<size_t>( "plimit" );
            particle_limit_f = true;
        }
        if ( this->hasOption( "pd" ) )
        {
            particle_density = this->optionValue<size_t>( "pd" );
            particle_density_f = true;
        }
        if ( this->hasOption( "hn" ) ) hostname = this->optionValue<std::string>( "hn" );
        if ( this->hasOption( "p" ) ) port = this->optionValue<size_t>( "p" );
        if ( this->hasOption( "pa" ) ) param_file = this->optionValue<std::string>( "pa" );
        if ( this->hasOption( "vin" ) )
        {
            std::string tmpstring = this->optionValue<std::string>( "vin" );
            // erase ".pfi"
            if ( tmpstring.find( ".pfi" ) != std::string::npos )
            {
                printf( "find pfi" );
                tmpstring.erase( tmpstring.find( ".pfi" ) );
                server_subvol_dir = tmpstring;
            }
            else
            {
                server_subvol_dir = this->optionValue<std::string>( "vin" );
            }
        }
// MODIFIED END   BY)M.Tanaka 2015.03.03
        if ( this->hasOption( "pout" ) )
        {
            pout_path_name = "PARTICLE_OUTDIR=";
            pout_path_name.append( this->optionValue<std::string>( "pout" ) );
        }
        else
        {
            pout_path_name = "PARTICLE_OUTDIR=";
            pout_path_name.append( "" );
        }
        if ( this->hasOption( "iout" ) )
        {
            image_base_name = this->optionValue<std::string>( "iout" );
        }
        else
        {
            image_base_name = "PBVR_image";
        }
#ifdef WIN32
        _putenv( pout_path_name.c_str() );
#else
        putenv( ( char* )pout_path_name.c_str() );
#endif
//      header = this->value<std::string>();
        int client_mode = 0;
        if ( this->hasOption( "vin" ) )
        {
            header = "";
            client_mode = 1;
        }
        else
        {
            header = "s";
        }

        // 初期化
        for ( int i = 0; i < 10; i++ )
        {
            pin_dir[i].clear();
        }
        // 統合粒子設定
        for ( int i = 0; i < 10; i++ )
        {
            char buf[64];
            sprintf( buf, "pin%d", i + 1 );
            if ( this->hasOption( buf ) )
            {
                kvs::File file_prefix( this->optionValue<std::string>( buf ) );
                kvs::Directory directory( file_prefix.pathName() );
                if ( !directory.isDirectory() )
                {
                    continue;
                }
                kvs::FileList::iterator bfile = directory.fileList().begin();
                kvs::FileList::iterator end = directory.fileList().end();
                int kvsmlFileNum = 0;

                while ( bfile != end )
                {
                    kvs::File f( bfile->fileName() );

                    if ( f.extension() == "kvsml" )
                    {
                        kvsmlFileNum++;
                    }
                    ++bfile;
                }

                if ( kvsmlFileNum == 0 )
                {
                    continue;
                }

                pin_dir[i].append( this->optionValue<std::string>( buf ) );
                client_mode = 2;
            }
        }
        if ( client_mode == 0 )
        {
            fprintf( stderr, "usage: %s -vin file_path [option]\n", argv[0] );
            fprintf( stderr, "         or\n" );
            fprintf( stderr, "       %s -pin* file_path [option]\n", argv[0] );
            exit( EXIT_FAILURE );
        }
    }
};

}
}

#endif
