#ifndef __KVS_VISCLIENT_COMMAND_H__
#define __KVS_VISCLIENT_COMMAND_H__

#include <vector>
#include <kvs/Mutex>

#include "PBVRParam.h"
#include "PBVRResult.h"
#include "ParticleServer.h"

namespace kvs
{
class TransferFunction;
class PointObject;
namespace glut
{
class Screen;
class Timer;
}
namespace visclient
{
struct PBVRParam; /* add 20140123 */
class ParticleServer;
class PVRenderer;
class Controller;
class TimerEvent;
class ProgressBar;
class TimeStepSlider;
class TimeSegmentMinSlider;
class TimeSegmentMaxSlider;
class TFE_main;
class ParticleMergerUI;

class Command
{
    kvs::Mutex mutex;

    struct SamplingParams
    {
        SamplingParams(
            const ParamExTransFunc& exTransFunc,
            const CropParam& crop,
            const std::string& dir ) :
            paramExTransFunc( exTransFunc ),
            paramCrop( crop ),
            inputdir( dir )
        {
        }

        const ParamExTransFunc paramExTransFunc;
        const CropParam paramCrop;
        const std::string inputdir;
    };
    const SamplingParams* lastSamplingParams;

private:
    int local_subpixel_level;

public:

    Command( ParticleServer* server );

    ~Command();

public:

    void startAbstractAnalysis();

    void startDetailedAnalysis();

    void changeShowHide();

    void preupdate();

    void update( PBVRParam& param, PBVRResult* result );

    void postupdate();

    void close_server();

    void getServerParticleInfomation( PBVRParam& param, PBVRResult* result );

    void initabstractParticles( PBVRParam& param, PBVRResult* result, int localminstep, int localmaxstep );

//    void generateAbstractParticles( PBVRParam& param, PBVRResult* result, int localminstep = -1, int localmaxstep = -1 );
    void generateAbstractParticles( PBVRParam& param, PBVRResult* result );

    void generateDetailedParticles();

    void reDraw();

    bool isSamplingParamsKeeped( const PBVRParam& param );

    void saveSamplingParams( const PBVRParam& param );

public:
    bool isUnderAnimation;
    bool isUnderCommunication;
    bool isShown;
    bool isImportTransferFunctionParam;
    bool isKeyFrameAnimation;
    int  stepKeyFrame;
    int  preKeyFrame;
    kvs::glut::Timer* glutTimer;
    ProgressBar* progress;
    TimeStepSlider* timeSlider;
    TimeSegmentMinSlider* minTimeSlider;
    TimeSegmentMaxSlider* maxTimeSlider;
    PBVRParam param;
    PBVRResult result;
    ParticleServer* server;
    PVRenderer* renderer;
    Controller* controller;
    kvs::glut::Screen* screen;
    TFE_main* extransfuncedit;
    kvs::glut::Screen* screen3;
    ParticleMergerUI* ParticleMergUI;

public:
    std::vector<kvs::PointObject*> abstractParticles;
    std::vector<kvs::PointObject*> detailedParticles;
    std::vector<kvs::PointObject*> serverParticles;

    size_t getUsingMemoryByKiloByte();
public:
    void lock();
    void unlock();

private:	// Rs[֎~D
    Command( const Command& );
    Command operator=( const Command& );

protected:
    void apply_variable_range( const VariableRange& range );

};

}
}

#endif
