#include "Label.h"

#include "Command.h"

#include "kvs/RendererBase"

using namespace kvs::visclient;

#if 1 // FEAST modify 20160128
extern int g_curStep;
#endif // FEAST modify 20160128 end


StepLabel::StepLabel( Command* command ): kvs::glut::Label( command->screen ), command( command )
{
    setMargin( 10 );
}

void StepLabel::screenUpdated( void )
{
    std::stringstream step;
    // FEAST modify 20160128
    //step << std::setprecision(4) << command->param.timeStep;
    if ( g_curStep < 0 )
        step << "---";
    else
        step << std::setprecision( 4 ) << g_curStep;
    // FEAST modify 20160128 end
    setText( std::string( "time step: " + step.str() ).c_str() );
}

SizeLabel::SizeLabel( Command* command ) : kvs::glut::Label( command->screen )
{
    setMargin( 10 );
}

void SizeLabel::screenUpdated( void )
{
    setText( std::string( "Data size: 4.2 GB/step" ).c_str() );
}

NodesLabel::NodesLabel( kvs::visclient::Command* command ): kvs::glut::Label( command->screen )
{
    setMargin( 10 );
}

void NodesLabel::screenUpdated( void )
{
    setText( std::string( "Nodes    :  26,047,774" ).c_str() );
}

ElementsLabel::ElementsLabel( kvs::visclient::Command* command ): kvs::glut::Label( command->screen )
{
    setMargin( 10 );
}

void ElementsLabel::screenUpdated( void )
{
    setText( std::string( "Elements : 126,713,831" ).c_str() );
}

FPSLabel::FPSLabel( kvs::ScreenBase* screen, const kvs::RendererBase* renderer ):
    kvs::glut::Label( screen ),
    renderer( renderer )
{
    setMargin( 10 );
}

void FPSLabel::screenUpdated( void )
{
    std::stringstream fps;
    fps << std::setprecision( 4 ) << renderer->timer().fps();
    setText( std::string( "fps: " + fps.str() ).c_str() );
}

void LegendBar::screenUpdated( void )
{
    kvs::visclient::ParamExTransFunc m_doc;
    kvs::ColorMap color_map;

    m_doc = m_command->param.paramExTransFunc;
    color_map = m_doc.transferFunction[select_tf].colorMap();

    setRange( m_doc.transferFunction[select_tf].ColorVarMin, m_doc.transferFunction[select_tf].ColorVarMax );
    setColorMap( color_map );
}

void LegendBar::select_tfr( int i )
{
    select_tf = i;
    screenResized();
}

void LegendBar::screenResized( void )
{
    if ( m_orientation == Horizontal )
    {
        setWidth( 200 );
        setHeight( 45 );
        setX( 15 );
        setY( 15 );
    }
    else
    {
        setWidth( 45 );
        setHeight( 200 );
        setX( 15 );
        setY( 15 );
    }
    screenUpdated();
}
