#include <algorithm>
#include <kvs/Directory>
#include <kvs/File>
#include <kvs/PointImporter>
#include <kvs/PointExporter>
#include <kvs/KVSMLObjectPoint>
#include "LegendpanelUI.h"
#include "KVSMLObjectPointWriter.h"
#include "Label.h"

namespace
{
kvs::visclient::LegendpanelUI* plegend_ui;

void cb_legend_display_val( int id )
{
    plegend_ui->set_display();
}
void cb_bar_display_val( int id )
{
    plegend_ui->set_Orientation();
}
void cb_caption_str( int id )
{
    plegend_ui->set_Caption();
}
void cb_Intervals_num( int id )
{
    plegend_ui->set_Division();
}
void cb_Intervals_color( int id )
{
    plegend_ui->show_LegendDivisionColor();
}
void cb_Intervals_thickness( int id )
{
    plegend_ui->set_Division_Thickness();
}
void cb_Framelines_color( int id )
{
    plegend_ui->show_LegendFrameColor();
}
void cb_Framelines_thickness( int id )
{
    plegend_ui->set_Franeline_Thickness();
}
void cb_selected_tf( int id )
{
    plegend_ui->select_tfr();
}
void cb_close( int id )
{
    plegend_ui->colsePanel();
}
} // end of namespace

namespace kvs
{
namespace visclient
{

LegendpanelUI::LegendpanelUI()
{
    legend_display_val[0] = 0;
}

LegendpanelUI::~LegendpanelUI()
{
}

void LegendpanelUI::createPanel( kvs::visclient::LegendBar* legend, kvs::ScreenBase* parent )
{
    m_screen = parent;
    plegend_ui = this;
    m_legend = legend;
    glui_legend_main = GLUI_Master.create_glui( "Legend panel", 0, 900, 100 );

    ch_b_legend = glui_legend_main->add_checkbox( "Display Legend", legend_display_val, 0, cb_legend_display_val );

    list_colormap = glui_legend_main->add_listbox( "Transfer Function Name", NULL, -1, cb_selected_tf );
    list_colormap->add_item( 0, "t1" );
    list_colormap->add_item( 1, "t2" );
    list_colormap->add_item( 2, "t3" );
    list_colormap->add_item( 3, "t4" );
    list_colormap->add_item( 4, "t5" );

    rdi_bar = glui_legend_main->add_radiogroup( NULL, -1, cb_bar_display_val );
    glui_legend_main->add_radiobutton_to_group( rdi_bar, "vertical" );
    glui_legend_main->add_radiobutton_to_group( rdi_bar, "horizontal" );

    strcpy( str_cpation, "" );
    tx_caption = glui_legend_main->add_edittext( "Caption", GLUI_EDITTEXT_TEXT, str_cpation, -1, cb_caption_str );
    tx_caption->set_w( 200 );

    pn_division = glui_legend_main->add_panel( "Division", GLUI_PANEL_EMBOSSED );
    tx_Intervals_num = glui_legend_main->add_edittext_to_panel( pn_division, "Intervals", GLUI_EDITTEXT_INT, NULL, -1, cb_Intervals_num );
    tx_Intervals_num->set_int_val( 5 );
    bt_Intervals_color = glui_legend_main->add_button_to_panel( pn_division, "Edit Color", -1, cb_Intervals_color );
    tx_Intervals_thickness = glui_legend_main->add_edittext_to_panel( pn_division, "Thickness", GLUI_EDITTEXT_INT, NULL, -1, cb_Intervals_thickness );
    tx_Intervals_thickness->set_int_val( 1 );

    pn_Frameline = glui_legend_main->add_panel( "Frameline", GLUI_PANEL_EMBOSSED );
    bt_Framelines_color = glui_legend_main->add_button_to_panel( pn_Frameline, "Edit Color", -1, cb_Framelines_color );
    bt_Frameline_thickness = glui_legend_main->add_edittext_to_panel( pn_Frameline, "Thickness", GLUI_EDITTEXT_INT, NULL, -1, cb_Framelines_thickness );
    bt_Frameline_thickness->set_int_val( 1 );

    glui_legend_main->add_statictext( "" );
    glui_legend_main->add_button( "Close", 0, cb_close );
    glui_legend_main->add_statictext( "" );

    // Set init Parameter
    select_tfr();
    set_display();
    set_Orientation();
    set_Division();
    set_Division_Thickness();
    set_Franeline_Thickness();
    set_Caption();

    glui_legend_main->hide();
    show_ui = false;

    m_lfc = new kvs::visclient::Legend_Frame_Color( this, m_screen );
    m_ldc = new kvs::visclient::Legend_Division_Color( this, m_screen );
}

bool LegendpanelUI::get_show_ui()
{
    return show_ui;
}

void LegendpanelUI::change_show_ui()
{
    show_ui = !show_ui;
}

void LegendpanelUI::select_tfr()
{
    m_legend->select_tfr( list_colormap->get_int_val() );
}

void LegendpanelUI::set_display()
{
    if ( legend_display_val[0] == 0 )
    {
        m_legend->hide();
    }
    else
    {
        m_legend->show();
    }
}

void LegendpanelUI::set_Orientation()
{
    if ( rdi_bar->get_int_val() == 0 )
    {
        //                m_legend->setOrientation( LegendBar::OrientationType::Vertical );
        m_legend->setOrientation( LegendBar::Vertical );
    }
    else
    {
        //                m_legend->setOrientation( LegendBar::OrientationType::Horizontal );
        m_legend->setOrientation( LegendBar::Horizontal );
    }
    m_legend->screenResized();
}

void LegendpanelUI::set_Division()
{
    int val = tx_Intervals_num->get_int_val();
    if ( val < 1 )
    {
        tx_Intervals_num->set_int_val( 1 );
    }
    m_legend->setNumberOfDivisions( val );
}

void LegendpanelUI::set_Division_Thickness()
{
    int val = tx_Intervals_thickness->get_int_val();
    if ( val < 0 )
    {
        tx_Intervals_thickness->set_int_val( 0 );
    }
    m_legend->setDivisionLineWidth( val );
}

void LegendpanelUI::set_Division_Color()
{
    const kvs::RGBColor crgb = m_ldc->color();
    m_legend->setDivisionLineColor( crgb );
}

void LegendpanelUI::set_Franeline_Thickness()
{
    int val = bt_Frameline_thickness->get_int_val();
    if ( val < 0 )
    {
        bt_Frameline_thickness->set_int_val( 0 );
    }
    m_legend->setBorderWidth( val );
}

void LegendpanelUI::set_Franeline_Color()
{
    const kvs::RGBColor crgb = m_lfc->color();
    m_legend->setBorderColor( crgb );
}

void LegendpanelUI::set_Caption()
{
    const std::string strval = std::string( str_cpation );
    m_legend->setCaption( strval );
}

void LegendpanelUI::colsePanel()
{
    show_ui = false;
    glui_legend_main->hide();
}

void LegendpanelUI::show_LegendFrameColor()
{
    m_lfc->show();
    m_lfc->redraw();
}

void LegendpanelUI::show_LegendDivisionColor()
{
    m_ldc->show();
    m_ldc->redraw();
}

} // namespace visclient
} // namespace kvs
