#include <sstream>
#include <algorithm>
#include <kvs/Directory>
#include <kvs/File>
#include <kvs/PointImporter>
#include <kvs/PointExporter>
#include <kvs/KVSMLObjectPoint>
#include "ParticleMerger.h"
#include "KVSMLObjectPointWriter.h"

namespace kvs
{
namespace visclient
{

ParticleMerger::ParticleMerger():
    m_initial_step( m_param.particles.size() ),
    m_final_step( m_param.particles.size() ),
    m_div( m_param.particles.size() ),
    m_fileprefix( m_param.particles.size() )
{
}

ParticleMerger::~ParticleMerger()
{
}

void ParticleMerger::setParam( const ParticleMergeParam& param, const size_t server_init_step, const size_t server_fin_step )
{
    size_t num = param.particles.size();
    m_lastparam = m_param;
    m_param = param;

    for ( size_t n = 0; n < num; n++ )
    {
        const std::string filepath = m_param.particles[n].filepath;
        const std::string lastfilepath = m_lastparam.particles[n].filepath;
        if ( filepath == "server" )
        {
            m_initial_step[n] = server_init_step;
            m_final_step[n] = server_fin_step;
        }
        else if ( filepath != lastfilepath )
        {
            std::cout << "FILE check " << filepath << std::endl;
            kvs::File file_prefix( filepath );
            // search final step
            kvs::Directory dir( file_prefix.pathName() );
            kvs::FileList pre_files = dir.fileList();
            kvs::FileList files( 0 );
            kvs::FileList::iterator fi;
            std::string prefix;
            size_t num_kvsml = 0;
            size_t step;
            size_t div = 0;
            size_t init_step = 0, fin_step = 0;
            std::string sstep, sdiv;
            std::stringstream ss;

            for ( fi = pre_files.begin(); fi != pre_files.end(); fi++ )
            {
                std::string bn = fi->baseName();
                size_t sep1 = bn.find( "_", 0 );
                prefix = bn.substr( 0, sep1 );
                if ( prefix == file_prefix.fileName() )
                    files.push_back( *fi );
            }

            for ( fi = files.begin(); fi != files.end(); fi++ )
            {
                std::string ext = fi->extension();
                if ( ext == "kvsml" )
                {
                    std::string bn = fi->baseName();
                    size_t sep1 = bn.find( "_", 0 );
                    size_t sep2 = bn.find( "_", sep1 + 1 );
                    size_t sep3 = bn.find( "_", sep2 + 1 );
                    prefix = bn.substr( 0, sep1 );
                    sstep = bn.substr( sep1 + 1, 5 );
                    sdiv  = bn.substr( sep3 + 1, 7 );
//                    std::cout << bn << " " << prefix << " " << sdiv << std::endl;

                    ss.clear();
                    ss << sstep << ' ' << sdiv;
                    ss >> step >> div;

                    init_step = ( num_kvsml ) ? std::min( init_step, step ) : step;
                    fin_step  = ( num_kvsml ) ? std::max( fin_step, step ) : step;

                    num_kvsml++;
                }
            }

            m_div[n] = div;
            m_initial_step[n] = init_step;
            m_final_step[n] = fin_step;
            m_fileprefix[n] = dir.directoryPath( true ) + dir.Separator() + prefix;

            if ( num_kvsml != 0 )
            {
                // 存在領域設定ファイルがあるか確認をする
                std::string filename( filepath );
//              filename.append( dir.Separator().c_str() );
//              filename.append( prefix.c_str() );
                filename.append( "_pfi_coords_minmax.txt" );
                kvs::File f( filename.c_str()  );
                if ( f.isExisted() )
                {
                    // ファイルがある
                    FILE* fp = NULL;
                    fp = fopen( filename.c_str(), "r" );
                    fscanf( fp, "%f %f %f %f %f %f",
                            &m_param.particles[n].xmin,
                            &m_param.particles[n].ymin,
                            &m_param.particles[n].zmin,
                            &m_param.particles[n].xmax,
                            &m_param.particles[n].ymax,
                            &m_param.particles[n].zmax );
                    if ( fp != NULL ) fclose( fp );
                }
                else
                {
                    // ファイルがない
                    // 最初のstepのファイルをopenして情報を得る
                    std::string prefix = m_fileprefix[n];
                    std::stringstream suffix;
                    suffix << '_' << std::setw( 5 ) << std::setfill( '0' ) << m_initial_step[n]
                           << '_' << std::setw( 7 ) << std::setfill( '0' ) << 1
                           << '_' << std::setw( 7 ) << std::setfill( '0' ) << m_div[n];
                    std::string filename = prefix + suffix.str() + ".kvsml";

                    const kvs::File file( filename );
                    kvs::PointObject* impobj = new kvs::PointImporter( filename );
                    const kvs::Vector3f& min_t = impobj->minObjectCoord();
                    const kvs::Vector3f& max_t = impobj->maxObjectCoord();
                    m_param.particles[n].xmin = min_t[0];
                    m_param.particles[n].ymin = min_t[1];
                    m_param.particles[n].zmin = min_t[2];
                    m_param.particles[n].xmax = max_t[0];
                    m_param.particles[n].ymax = max_t[1];
                    m_param.particles[n].zmax = max_t[2];
                    delete impobj;
                }
            }
        }
    }
}

bool ParticleMerger::CalcExternalCoords( float* crd )
{
    bool cf = false;
    size_t num = m_param.particles.size();
    for ( size_t n = 1; n < num; n++ )
    {
        if ( !m_param.particles[n].enable ) continue;
        if ( cf )
        {
            xmin = std::min( xmin, m_param.particles[n].xmin );
            ymin = std::min( ymin, m_param.particles[n].ymin );
            zmin = std::min( zmin, m_param.particles[n].zmin );
            xmax = std::max( xmax, m_param.particles[n].xmax );
            ymax = std::max( ymax, m_param.particles[n].ymax );
            zmax = std::max( zmax, m_param.particles[n].zmax );
        }
        else
        {
            cf = true;
            xmin = m_param.particles[n].xmin;
            ymin = m_param.particles[n].ymin;
            zmin = m_param.particles[n].zmin;
            xmax = m_param.particles[n].xmax;
            ymax = m_param.particles[n].ymax;
            zmax = m_param.particles[n].zmax;
        }
    }
    if ( cf )
    {
        crd[0] = xmin;
        crd[1] = ymin;
        crd[2] = zmin;
        crd[3] = xmax;
        crd[4] = ymax;
        crd[5] = zmax;
    }
    return cf;
}

void ParticleMerger::setenable( int i )
{
    m_param.particles[i].enable = true;
}

size_t ParticleMerger::getMergedInitialTimeStep()
{
    size_t init_step = getMergedFinalTimeStep();

    size_t num = m_param.particles.size();
    for ( size_t n = 0; n < num; n++ )
    {
        if ( !m_param.particles[n].enable ) continue;
        init_step = std::min( init_step, m_initial_step[n] );
    }

    return init_step;
}

size_t ParticleMerger::getMergedFinalTimeStep()
{
    size_t fin_step = 0;

    size_t num = m_param.particles.size();
    for ( size_t n = 0; n < num; n++ )
    {
        if ( !m_param.particles[n].enable ) continue;
        fin_step = std::max( fin_step, m_final_step[n] );
    }

    return fin_step;
}

kvs::PointObject* ParticleMerger::doMerge(
    const kvs::PointObject* server_particle,
    const size_t step )
{
    kvs::PointObject* obj = new kvs::PointObject();
    size_t num = m_param.particles.size();

    for ( size_t n = 0; n < num; n++ )
    {
        if ( !m_param.particles[n].enable ) continue;
        if ( ( step < m_initial_step[n] ) && ( !m_param.particles[n].keep_initial_step ) ) continue;
        if ( ( step > m_final_step[n] )   && ( !m_param.particles[n].keep_final_step )  ) continue;

        if ( server_particle && ( m_param.particles[n].filepath == "server" ) )
        {
            ( *obj ) += *server_particle;
            continue;
        }

        size_t filestep = std::max( std::min( step, m_final_step[n] ), m_initial_step[n] );

        for ( size_t m = 0; m < m_div[n]; m++ )
        {
            std::string prefix = m_fileprefix[n];
            std::stringstream suffix;
            suffix << '_' << std::setw( 5 ) << std::setfill( '0' ) << filestep
                   << '_' << std::setw( 7 ) << std::setfill( '0' ) << m + 1
                   << '_' << std::setw( 7 ) << std::setfill( '0' ) << m_div[n];
            std::string filename = prefix + suffix.str() + ".kvsml";
//          std::cout << filename << std::endl;

            const kvs::File file( filename );
            kvs::PointObject* impobj = new kvs::PointImporter( filename );
            size_t nmemb = impobj->coords().size();

            if ( nmemb != 0 )
            {
                ( *obj ) += *impobj;
            }
            delete impobj;
        }
    }

    if ( m_param.doExport )
    {
        // Export merged PointObject to kvsml file

        std::stringstream suffix;
        suffix << '_' << std::setw( 5 ) << std::setfill( '0' ) << step
               << '_' << std::setw( 7 ) << std::setfill( '0' ) << 1
               << '_' << std::setw( 7 ) << std::setfill( '0' ) << 1;

        std::string basename = m_param.export_filepath + suffix.str();

        std::cout << basename << std::endl;
        KVSMLObjectPointWriter( obj, basename );
    }

    return obj;
}

} // namespace visclient
} // namespace kvs

