/*****************************************************************************/
/**
 *  @file   TFE_Opacity_1.h
 *  @author Naohisa Sakamoto
 */
/*----------------------------------------------------------------------------
 *
 *  Copyright (c) Visualization Laboratory, Kyoto University.
 *  All rights reserved.
 *  See http://www.viz.media.kyoto-u.ac.jp/kvs/copyright/ for details.
 *
 *  $Id: TFE_Opacity_1.h 634 2010-10-13 07:04:05Z naohisa.sakamoto $
 */
/*****************************************************************************/
#ifndef KVS__GLUT__TFE_OPACITY_1_H_INCLUDE
#define KVS__GLUT__TFE_OPACITY_1_H_INCLUDE

#include <string>
#include <kvs/ClassName>
#include <kvs/OpenGL>
#include <kvs/RGBColor>
#include <kvs/RGBAColor>
#include <kvs/Vector2>
#include <kvs/TransferFunction>
#include <kvs/Texture1D>
#include <kvs/Texture2D>
#include <kvs/MouseReleaseEventListener>
#include <kvs/glut/Screen>
#include <kvs/glut/PushButton>
#include <kvs/glut/Text>
#include <kvs/glut/ColorPalette>
#include <kvs/glut/ColorMapPalette>
#include <kvs/glut/OpacityMapPalette>
#include <kvs/glut/Histogram>


namespace kvs
{

namespace visclient
{

class TFE_main;

/*===========================================================================*/
/**
 *  @brief  Transfer function editor class.
 */
/*===========================================================================*/
class TFE_Opacity_1 : public kvs::glut::Screen
{
    kvsClassName( kvs::visclient::TFE_Opacity_1 );

public:

    typedef kvs::glut::Screen SuperClass;

private:

    class StackEvent;

    std::deque<kvs::TransferFunction> m_undo_stack;     ///< undo stack
    std::deque<kvs::TransferFunction> m_redo_stack;     ///< redo stack
    StackEvent*                       m_stack_event;    ///< stack event
    size_t                            m_max_stack_size; ///< max. stack size

protected:

    bool m_visible;
    size_t m_resolution;

    kvs::ScreenBase*              m_screen;              ///< pointer to screen
    kvs::glut::OpacityMapPalette* m_opacity_map_palette; ///< opacity map palette
    kvs::glut::PushButton*        m_reset_button;        ///< reset button
    kvs::glut::PushButton*        m_undo_button;         ///< undo button
    kvs::glut::PushButton*        m_redo_button;         ///< redo button
    kvs::glut::PushButton*        m_save_button;         ///< save button
    kvs::glut::PushButton*        m_cancel_button;       ///< cancel button
    kvs::TransferFunction m_initial_transfer_function;   ///< initial transfer function
    kvs::Real32           m_min_value; ///< min value
    kvs::Real32           m_max_value; ///< max value

    kvs::visclient::TFE_main* m_parent;

public:

    TFE_Opacity_1( kvs::visclient::TFE_main* parent = 0 );

    virtual ~TFE_Opacity_1( void );

public:

    void setResolution( size_t resolution );

public:

    int show( void );

    virtual void hide( void );

public:

    kvs::ScreenBase* screen( void );

    const kvs::glut::OpacityMapPalette* opacityMapPalette( void ) const;

    const kvs::OpacityMap opacityMap( void ) const;

    const kvs::TransferFunction transferFunction( void ) const;

    const size_t undoStackSize( void ) const;

    const size_t redoStackSize( void ) const;

    const size_t maxStackSize( void ) const;

public:

    void setTransferFunction( const kvs::TransferFunction& transfer_function );

    void setVolumeObject( const kvs::VolumeObjectBase* object );

    void setMaxStackSize( const size_t stack_size );

public:

    virtual void reset( void );

    virtual void undo( void );

    virtual void redo( void );

    virtual void save( void );

    virtual void cancel( void );
};

/*===========================================================================*/
/**
 *  @brief  Stack event for undo/redo.
 */
/*===========================================================================*/
class TFE_Opacity_1::StackEvent : public kvs::MouseReleaseEventListener
{
    kvs::visclient::TFE_Opacity_1* m_editor;

public:

    StackEvent( kvs::visclient::TFE_Opacity_1* editor );

    void update( kvs::MouseEvent* event );
};

} // end of namespace visclient

} // end of namespace kvs

#endif // KVS__GLUT__TFE_OPACITY_1_H_INCLUDE
