/*****************************************************************************/
/**
 *  @file   TFE_Opacity_3.h
 *  @author Naohisa Sakamoto
 */
/*----------------------------------------------------------------------------
 *
 *  Copyright (c) Visualization Laboratory, Kyoto University.
 *  All rights reserved.
 *  See http://www.viz.media.kyoto-u.ac.jp/kvs/copyright/ for details.
 *
 *  $Id: TFE_Opacity_3.h 634 2010-10-13 07:04:05Z naohisa.sakamoto $
 */
/*****************************************************************************/
#ifndef KVS__GLUT__TFE_OPACITY_3_H_INCLUDE
#define KVS__GLUT__TFE_OPACITY_3_H_INCLUDE

#include <string>
#include <kvs/ClassName>
#include <kvs/OpenGL>
#include <kvs/RGBColor>
#include <kvs/RGBAColor>
#include <kvs/Vector2>
#include <kvs/TransferFunction>
#include <kvs/Texture1D>
#include <kvs/Texture2D>
#include <kvs/MouseReleaseEventListener>
#include <kvs/glut/Screen>
#include <kvs/glut/PushButton>
#include <kvs/glut/Text>
#include <kvs/glut/OpacityMapPalette>

#include "GL/glui.h"

namespace kvs
{

namespace visclient
{

class TFE_main;

/*===========================================================================*/
/**
 *  @brief  Transfer function editor class.
 */
/*===========================================================================*/
class TFE_Opacity_3 : public kvs::glut::Screen
{
    kvsClassName( kvs::visclient::TFE_Opacity_3 );

public:

    typedef kvs::glut::Screen SuperClass;

protected:

    bool m_visible;
    size_t m_resolution;

    kvs::ScreenBase*              m_screen;              ///< pointer to screen
    kvs::glut::OpacityMapPalette* m_opacity_map_palette; ///< opacity map palette
    kvs::TransferFunction m_initial_transfer_function;   ///< initial transfer function
    kvs::Real32           m_min_value; ///< min value
    kvs::Real32           m_max_value; ///< max value

    kvs::visclient::TFE_main* m_parent;

    GLUI* m_glui_text_win;
    GLUI* m_glui_button_win;
    GLUI_EditText* m_glui_edit_c[10];
    GLUI_EditText* m_glui_edit_o[10];

public:

    TFE_Opacity_3( kvs::visclient::TFE_main* parent = 0 );

    virtual ~TFE_Opacity_3( void );

public:

    void setResolution( size_t resolution );

public:

    kvs::ScreenBase* screen( void );

    const kvs::glut::OpacityMapPalette* opacityMapPalette( void ) const;

    const kvs::OpacityMap opacityMap( void ) const;

    const kvs::TransferFunction transferFunction( void ) const;

public:

    int show( void );

    virtual void hide( void );

public:

    void setTransferFunction( const kvs::TransferFunction& transfer_function );

public:

    void setControlPointValue( size_t n,  float cp, float opacity );

public:

    virtual void update( void );

    virtual void save( void );

    virtual void cancel( void );
};

} // end of namespace visclient

} // end of namespace kvs

#endif // KVS__GLUT__TFE_OPACITY_3_H_INCLUDE
