/*****************************************************************************/
/**
 *  @file   TFE_main.h
 *  @author Naohisa Sakamoto
 */
/*----------------------------------------------------------------------------
 *
 *  Copyright (c) Visualization Laboratory, Kyoto University.
 *  All rights reserved.
 *  See http://www.viz.media.kyoto-u.ac.jp/kvs/copyright/ for details.
 *
 *  $Id: TFE_main.h 634 2010-10-13 07:04:05Z naohisa.sakamoto $
 */
/*****************************************************************************/
#ifndef KVS__GLUT__TFE_MAIN_H_INCLUDE
#define KVS__GLUT__TFE_MAIN_H_INCLUDE

#include <string>
#include <kvs/ClassName>
#include <kvs/OpenGL>
#include <kvs/RGBColor>
#include <kvs/RGBAColor>
#include <kvs/Vector2>
#include <kvs/TransferFunction>
#include <kvs/Texture1D>
#include <kvs/Texture2D>
#include <kvs/MouseReleaseEventListener>
#include <kvs/glut/Screen>
#include <kvs/glut/PushButton>
#include <kvs/glut/Text>
#include <kvs/glut/ColorPalette>
#include <kvs/glut/ColorMapPalette>
#include <kvs/glut/OpacityMapPalette>
#include "Histogram.h"

#include "GL/glui.h"

#include "TFE_Color_1.h"
#include "TFE_Color_2.h"
#include "TFE_Color_3.h"
#include "TFE_Color_4.h"
#include "TFE_Opacity_1.h"
#include "TFE_Opacity_2.h"
#include "TFE_Opacity_3.h"

#include "ParamExTransFunc.h"
#include "Command.h"

namespace kvs
{

namespace visclient
{

/*===========================================================================*/
/**
 *  @brief  Transfer function editor class.
 */
/*===========================================================================*/
class TFE_main : public kvs::glut::Screen
{
    kvsClassName( kvs::visclient::TFE_main );

public:

    typedef kvs::glut::Screen SuperClass;

    typedef std::map<int, kvs::TransferFunction> TransferFunctionSet;

protected:

    Command* m_command;

    ParamExTransFunc m_doc;
    ParamExTransFunc m_doc_initial;
    int m_current_tf;

    kvs::ScreenBase*              m_screen;              ///< pointer to screen
    kvs::glut::ColorMapPalette*   m_color_map_palette;   ///< color map palette
    kvs::glut::OpacityMapPalette* m_opacity_map_palette; ///< opacity map palette
    kvs::visclient::Histogram*    m_color_histogram;     ///< color histogram
    kvs::visclient::Histogram*    m_opacity_histogram;   ///< opacity histogram
    kvs::glut::PushButton*        m_reset_button;        ///< reset button
    kvs::glut::PushButton*        m_apply_button;        ///< apply button
    kvs::Real32           m_min_value; ///< min value
    kvs::Real32           m_max_value; ///< max value

    kvs::visclient::TFE_Color_1* m_tfe_c1;
    kvs::visclient::TFE_Color_2* m_tfe_c2;
    kvs::visclient::TFE_Color_3* m_tfe_c3;
    kvs::visclient::TFE_Color_4* m_tfe_c4;
    kvs::visclient::TFE_Opacity_1* m_tfe_o1;
    kvs::visclient::TFE_Opacity_2* m_tfe_o2;
    kvs::visclient::TFE_Opacity_3* m_tfe_o3;

    GLUI* m_glui_common;
    GLUI* m_glui_tferng;
    GLUI* m_glui_tffile;
    GLUI* m_glui_tffctl;

    GLUI_EditText* m_glui_resolution;
    GLUI_EditText* m_glui_tf_synth;
    GLUI_Listbox*  m_glui_tf_name;
    GLUI_EditText* m_glui_tf_var_c;
    GLUI_EditText* m_glui_tf_min_c;
    GLUI_EditText* m_glui_tf_max_c;
    GLUI_EditText* m_glui_tf_var_o;
    GLUI_EditText* m_glui_tf_min_o;
    GLUI_EditText* m_glui_tf_max_o;
    GLUI_EditText* m_glui_tf_path;
    GLUI_StaticText* m_glui_rng_min_c;
    GLUI_StaticText* m_glui_rng_max_c;
    GLUI_StaticText* m_glui_rng_min_o;
    GLUI_StaticText* m_glui_rng_max_o;

    bool show_tfe;

public:

    TFE_main( Command* command, kvs::ScreenBase* parent = 0 );

    virtual ~TFE_main( void );

public:

    int show( void );

    virtual void hide( void );

    bool get_show_tfe( void );

public:

    kvs::ScreenBase* screen( void );

    const kvs::glut::ColorMapPalette* colorMapPalette( void ) const;

    const kvs::glut::OpacityMapPalette* opacityMapPalette( void ) const;

    const kvs::ColorMap colorMap( void ) const;

    const kvs::OpacityMap opacityMap( void ) const;

    const kvs::TransferFunction transferFunction( void ) const;

    kvs::visclient::ParamExTransFunc& doc( void );

    const kvs::visclient::ParamExTransFunc& doc( void ) const;

public:

    void setVolumeObject( const kvs::VolumeObjectBase* object );

    void exportFile( const std::string& fname, bool addition = false );

    void importFile( const std::string& fname );
    void importFileWithInit( const std::string& fname );

public:

    virtual void resolution( void );

    virtual void function_name( void );

    virtual void function_variable( void );

    virtual void reset( void );

    virtual void apply( void );

    virtual void import_file( void );

    virtual void export_file( void );

    virtual void show_tfe_c1( void );

    virtual void show_tfe_c2( void );

    virtual void show_tfe_c3( void );

    virtual void show_tfe_c4( void );

    virtual void show_tfe_o1( void );

    virtual void show_tfe_o2( void );

    virtual void show_tfe_o3( void );

    virtual void save_tfe_c1( void );

    virtual void save_tfe_c2( void );

    virtual void save_tfe_c3( void );

    virtual void save_tfe_c4( void );

    virtual void save_tfe_o1( void );

    virtual void save_tfe_o2( void );

    virtual void save_tfe_o3( void );

public:

    void display_transferFunction( const int n );

    void update_range_view( void );

    void update_range_view( const int n );

    void update_range_edit( void );

    void update_range_edit( const int n );

};

} // end of namespace visclient

} // end of namespace kvs

#endif // KVS__GLUT__TFE_MAIN_H_INCLUDE
