#include "PointObjectGenerator.h"
//#include <sys/time.h>
#include "TransferFunction.h"
#include "UnstructuredVolumeObject.h"
#include "UnstructuredVolumeImporter.h"
#include "CellByCellUniformSampling.h"
#include "CellByCellRejectionSampling.h"
#include "CellByCellMetropolisSampling.h"
//#include "CellByCellLayeredSampling.h"
#include <kvs/Camera>
#include "CellByCellUniformSampling.h"
#include "CellByCellRejectionSampling.h"
#include "CellByCellMetropolisSampling.h"
#if 0 //TEST_DELETE
#include <kvs/TestVolume>
#include <kvs/FrontSTRFileReader>
#endif
#include <kvs/AVSUcd>
#include "common.h"
#include <kvs/ValueArray>

#include "Argument.h"
#ifdef KMATH
#include <kmath_random.h>
#endif

#include "timer_simple.h"

using namespace pbvr;
#ifdef KMATH
extern KMATH_Random km_random;
#endif

void PointObjectGenerator::createTestObject( const size_t howMany )
{
    delete object;
    kvs::ValueArray<float> points( howMany * 3 );
    double* rv = new double[howMany * 3 * 2];
#ifdef KMATH
    #pragma omp critical(random)
    {
        km_random.get( rv, howMany * 3 * 2 );
    }
    for ( size_t i = 0; i < howMany * 3 * 2; ++i )
    {
        rv[i] -= 1.0;
    }
#endif
    for ( size_t i = 0; i < howMany; ++i )
    {
#ifndef KMATH
        points[i * 3] = ( rand() % 1000 ) / 1000.0f * 2.0f;
        points[i * 3 + 1] = ( rand() % 1000 ) / 1000.0f * 2.0f;
        points[i * 3 + 2] = ( rand() % 1000 ) / 1000.0f * 2.0f;
#else
        points[i * 3]   = ( static_cast<int>( rv[i * 3  ] * RAND_MAX ) % 1000 ) / 1000.0f * 2.0f;
        points[i * 3 + 1] = ( static_cast<int>( rv[i * 3 + 1] * RAND_MAX ) % 1000 ) / 1000.0f * 2.0f;
        points[i * 3 + 2] = ( static_cast<int>( rv[i * 3 + 2] * RAND_MAX ) % 1000 ) / 1000.0f * 2.0f;
#endif
    }
    kvs::ValueArray<kvs::UInt8> colors( howMany * 3 );
    for ( size_t i = 0; i < howMany; ++i )
    {
#ifndef KMATH
        colors[i * 3] = ( rand() % 1000 ) / 1000.0f * 255;
        colors[i * 3 + 1] = ( rand() % 1000 ) / 1000.0f * 255;
        colors[i * 3 + 2] = ( rand() % 1000 ) / 1000.0f * 255;;
#else
        colors[i * 3]   = ( static_cast<int>( rv[howMany + i * 3  ] * RAND_MAX ) % 1000 ) / 1000.0f * 255;
        colors[i * 3 + 1] = ( static_cast<int>( rv[howMany + i * 3 + 1] * RAND_MAX ) % 1000 ) / 1000.0f * 255;
        colors[i * 3 + 2] = ( static_cast<int>( rv[howMany + i * 3 + 2] * RAND_MAX ) % 1000 ) / 1000.0f * 255;;
#endif
    }
    delete[] rv;
    object = new pbvr::PointObject( points, colors, 30 );
}

void PointObjectGenerator::createPyramidPointObject( Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step )
{
#if 0 //TEST_DELETE
    delete object;
    pbvr::UnstructuredVolumeObject* volume;
    GenerateVolumeData gen_vol( 128 );

    std::cout << "gen pyramid data." << std::endl;
    gen_vol.pyramid( 128 );
    volume = gen_vol.m_volume;

    std::cout << *volume << std::endl;

    object = sampling( param, camera, volume, subpixel_level, sampling_step );
#endif
}

void PointObjectGenerator::createQuadraticHexahedra( Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step )
{
#if 0 //TEST_DELETE
    delete object;
    pbvr::UnstructuredVolumeObject* volume;
    GenerateVolumeData gen_vol( 128 );

    std::cout << "gen quadratic hexahedra data." << std::endl;
    gen_vol.quadraticHexahedra( 128 );
    volume = gen_vol.m_volume;

    std::cout << *volume << std::endl;

    object = sampling( param, camera, volume, subpixel_level, sampling_step );
#endif
}

void PointObjectGenerator::createHTTR(
    Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step, const size_t step )
{
    delete object;

    char filename[256];
    sprintf( filename, "%s/binary0%03d.dat", param.input_data.c_str(), step );
    //sprintf(filename,"%s/divide0%03d-04639.dat", param.input_data.c_str(), step);
    std::cout << filename << std::endl;
    pbvr::UnstructuredVolumeObject*  volume = read_avsucd( filename, param.component_Id );

    std::cout << *volume << std::endl;
    object = sampling( param, camera, volume, subpixel_level, sampling_step );
    delete volume;
}

void PointObjectGenerator::createGT5D(
    Argument& param, kvs::Camera& camera, size_t subpixel_level,
    float sampling_step, pbvr::UnstructuredVolumeObject* volume )
{
    delete object;
    std::cout << "start particle generation\n";
    std::cout << *volume << std::endl;
    object = sampling( param, camera, volume, subpixel_level, sampling_step );
    std::cout << "end particle generation\n";
}

void PointObjectGenerator::createGT5D_full(
    Argument& param, kvs::Camera& camera, size_t subpixel_level,
    float sampling_step, pbvr::UnstructuredVolumeObject* volume )
{
    delete object;
    pbvr::TransferFunction& tf = param.transfer_function;
    object = new pbvr::CellByCellUniformSampling
    ( &camera, volume, subpixel_level, sampling_step, tf,
      param.transfunc_synthesizer, param.normal_ingred, param.crop, true );
}
/**************************
    int mpi_size = 1;
    int ts_file_no = -1;
    int ts_file_max_no = 0;

#ifdef ENABLE_MPI
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
    MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
    if(mpi_rank == 0){
#endif
        kvs::test::AVSField* field =
            new kvs::test::AVSField( header, coord, variable );
        pbvr::UnstructuredVolumeObject* volume = new pbvr::UnstructuredVolumeObject;
        for( size_t i = 0; i < GT5D_NUM; i++ )
        {
#ifdef ENABLE_MPI
            if((time_series==0)||(mpi_rank==i%mpi_size)){
                if(time_series==1) MPI_Barrier(MPI_COMM_WORLD);
#endif
                while(field->currentTimeStep() != i){
                    pbvr::PointObject* pobject;
                }
                field->progress();
                Import( volume, field, i ,0);
                switch (param.sampling_method) {
                case 'u':
                    std::cout << "Uniform sampling" << std::endl;
                    pobject = new kvs::test::CellByCellUniformSampling(
                        screen.camera(), volume, subpixel_level, sampling_step, tfunc);
                    -->calling();
                    {
                        int sampling_method = MPIController::UNIFORM_SAMPLING;
                        MPI_Bcast(&sampling_method, 1, MPI_INT, 0, MPI_COMM_WORLD);
                        DistributedUniformSampling::generate_particles_master_gpu(
                            m_coords etc..);
                        -->calling();
                        {
                            DistributedUniformSampling::generate_particles_core_gpu(
                                m_coords etc..);
                        }
                    }
                }
            }
        }
#ifdef ENABLE_MPI
//          if(time_series==1) MPI_Barrier(MPI_COMM_WORLD);
        int exit_command = MPIController::EXIT_PROCESS;
        MPI_Bcast(&exit_command, 1, MPI_INT, 0, MPI_COMM_WORLD);
        for(int i=1;i<mpi_size;i++){
            MPI_Recv(&TIMER[i][0], 100, MPI_REAL, i, 0, MPI_COMM_WORLD,
                     MPI_STATUS_IGNORE);
        }
#endif
        delete volume;
        field->close();
        delete field;
#ifdef ENABLE_MPI
        MPI_Finalize();
#endif
#ifdef ENABLE_MPI
    } // end of if(mpi_rank == 0 )
    else{ // ELSE (mpi_rank !=0)
        init_tinymt();
        if(time_series==1){
            kvs::test::AVSField* field =
                new kvs::test::AVSField( header, coord, variable );
            pbvr::UnstructuredVolumeObject* volume = new pbvr::UnstructuredVolumeObject;
            ts_file_max_no = ((GT5D_NUM-1)/mpi_size+1) * mpi_size;
            size_t total_particles = 0;
            for (int i = 0; i < ts_file_max_no; i++) {
                if(mpi_rank == i%mpi_size){
                    MPI_Barrier(MPI_COMM_WORLD);
                    ts_file_no = std::min(GT5D_NUM-1,i);
                    while(field->currentTimeStep() != ts_file_no){
                        field->progress();
                    }
                    field->progress();
                    Import( volume, field, ts_file_no, mpi_rank);
                    int sampling_method = 0;
                    MPI_Bcast(&sampling_method, 1, MPI_INT, 0, MPI_COMM_WORLD);
                    if(sampling_method == MPIController::EXIT_PROCESS){
                        break;
                    }
                    switch(sampling_method){
                    case MPIController::UNIFORM_SAMPLING:
                        DistributedUniformSampling::generate_particles_slave_ts_gpu(vol);
                        -->calling();
                        {
                            DistributedUniformSampling::generate_particles_core_gpu(
                                volume,NULL);
                        }
                    }
                    timer.stop();
                    std::cout<<mpi_rank<<":sampling time:"<<timer.msec()<<std::endl<<std::endl;
                }
            }

        }
        else{ // time_series==0
            for(;;){
                int sampling_method = 0;
                MPI_Bcast(&sampling_method, 1, MPI_INT, 0, MPI_COMM_WORLD);
                if(sampling_method == MPIController::EXIT_PROCESS){
                    break;
                }
                switch(sampling_method){
                case MPIController::UNIFORM_SAMPLING:
                    kvs::test::DistributedUniformSampling::generate_particles_slave_gpu();
                    -->calling();
                    {
                        DistributedUniformSampling::generate_particles_core_gpu(
                            NULL,NULL);
                    }
                }
            }
            MPI_Send(&TIMER[mpi_rank][0], 100, MPI_REAL, 0, 0, MPI_COMM_WORLD);
        } // end of else( time_series==0)
    } // end of else (mpi_rank!=0)
    MPI_Finalize();
*************************/

void PointObjectGenerator::createFromFile( Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step )
{
//FJ_TIMER_KAWAMURA
    TIMER_STA( 260 );
//FJ_TIMER_KAWAMURA

    delete object;
    pbvr::UnstructuredVolumeObject* volume;
    volume = new pbvr::UnstructuredVolumeImporter( param.input_data );

//FJ_TIMER_KAWAMURA
    TIMER_END( 260 );
//FJ_TIMER_KAWAMURA

    volume->setMinMaxValues( m_fi->minValue, m_fi->maxValue );
    volume->setMinMaxObjectCoords( m_fi->minObjectCoord, m_fi->maxObjectCoord );
    volume->setMinMaxExternalCoords( m_fi->minObjectCoord, m_fi->maxObjectCoord );

    std::cout << *volume << std::endl;
    std::cout << "min:" << volume->minObjectCoord() << ", max:" << volume->maxObjectCoord() << std::endl;
    std::cout << "min:" << volume->minExternalCoord() << ", max:" << volume->maxExternalCoord() << std::endl;

    object = sampling( param, camera, volume, subpixel_level, sampling_step );
    delete volume;
}

void PointObjectGenerator::createFromFile( Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step, int st, int vl )
{
    TIMER_STA( 260 );
    delete object;
    pbvr::UnstructuredVolumeObject* volume;

    // volume = new pbvr::UnstructuredVolumeImporter( param.input_data );
    volume = new pbvr::UnstructuredVolumeImporter( param.input_data_base, m_fi->fileType, st, vl );

    TIMER_END( 260 );

    volume->setMinMaxValues( m_fi->minValue, m_fi->maxValue );
    volume->setMinMaxObjectCoords( m_fi->minObjectCoord, m_fi->maxObjectCoord );
    volume->setMinMaxExternalCoords( m_fi->minObjectCoord, m_fi->maxObjectCoord );

    std::cout << *volume << std::endl;
    std::cout << "min:" << volume->minObjectCoord() << ", max:" << volume->maxObjectCoord() << std::endl;
    std::cout << "min:" << volume->minExternalCoord() << ", max:" << volume->maxExternalCoord() << std::endl;

    object = sampling( param, camera, volume, subpixel_level, sampling_step );
    delete volume;
}

std::string PointObjectGenerator::getErrorMessage( const size_t maxMemory ) const
{
    std::string errorMessage( "" );
    const size_t totalMemory = sizeof( float ) * object->nvertices() + sizeof( float ) * object->nnormals() + sizeof( char ) * object->ncolors();
    if ( totalMemory > maxMemory )
    {
        char ms[512];
        sprintf( ms, "Memory Error: TotalMemory = %zd, MaxMemory = %zd\n", totalMemory, maxMemory );
        errorMessage += ms;
    }
    return errorMessage;
}

pbvr::PointObject* PointObjectGenerator::sampling( Argument& param, kvs::Camera& camera, pbvr::VolumeObjectBase* volume, size_t subpixel_level, float sampling_step )
{
    pbvr::TransferFunction& tf = param.transfer_function;

//    tf.setRange(volume);
//    param.transfunc_synthesizer->transfunc_map()["t1"] = tf;
//    param.transfunc_synthesizer->create();

    // volume calculate test.
#if 0 //TEST_DELETE
    if ( param.test_volume )
    {
        return new kvs::TestVolume( &camera, volume, subpixel_level, sampling_step, tf );
    }
#endif

#ifdef CPU_SAMPLING_TIME
    std::cout << std::endl << "CPU - ";
#else
    std::cout << std::endl << "GPU - ";
#endif
    switch ( param.sampling_method )
    {
    case 'u':
        std::cout << "Uniform sampling" << std::endl;
        return new pbvr::CellByCellUniformSampling( &camera, volume, subpixel_level, sampling_step, tf,
                                                   param.transfunc_synthesizer, param.normal_ingred, param.crop,
                                                   param.particle_density, param.batch );
    case 'r':
        std::cout << "Rejection sampling" << std::endl;
        return new pbvr::CellByCellRejectionSampling( &camera, volume, subpixel_level, sampling_step, tf,
                                                     param.transfunc_synthesizer, param.crop,
                                                     param.particle_density, param.batch );
    case 'm':
        std::cout << "Metropolis sampling" << std::endl;
        return new pbvr::CellByCellMetropolisSampling( &camera, volume, subpixel_level, sampling_step, tf,
                                                      param.transfunc_synthesizer, param.normal_ingred, param.crop,
                                                      param.particle_density, param.batch );
    default:
        std::cout << "Unknown sampling method:" << param.sampling_method << std::endl;
        exit( 1 );
        return 0;
    }
}

pbvr::UnstructuredVolumeObject* PointObjectGenerator::read_avsucd( char* filename, const int componentId )
{
    kvs::AVSUcd* ucd = new kvs::AVSUcd( filename, 0, componentId );

    pbvr::UnstructuredVolumeObject* volume = new pbvr::UnstructuredVolumeObject();
    volume->setVeclen( ucd->veclens()[ ucd->componentID() ] );
    volume->setNNodes( ucd->nnodes() );
    volume->setNCells( ucd->nelements() );
    //消費するビデオメモリを抑えるため.
    volume->setCellType( pbvr::UnstructuredVolumeObject::Tetrahedra );
    volume->setCoords( ucd->coords() );
    volume->setConnections( ucd->connections() );
    volume->setValues( kvs::AnyValueArray( ucd->values() ) );
    volume->updateMinMaxCoords();
    volume->updateMinMaxValues();

    delete ucd;
    return volume;
}
