#ifndef __POINT_OBJECT_GENERATOR_H__
#define __POINT_OBJECT_GENERATOR_H__

#include <vector>
#include <string>
#include "UnstructuredVolumeObject.h"
#include "FilterInfo.h"

class Argument;
namespace kvs
{
class Camera;
}

namespace pbvr
{
class PointObject;
class VolumeObjectBase;
class UnstructuredVolumeObject;

class PointObjectGenerator
{
private:
    pbvr::PointObject* object;
    const FilterInfo*   m_fi;

public:

    PointObjectGenerator() : object( 0 ) {};

    void createTestObject( const size_t howMany );

    void createPyramidPointObject(
        Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step );

    void createQuadraticHexahedra(
        Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step );

    void createHTTR(
        Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step, size_t step );

    void createGT5D(
        Argument& param, kvs::Camera& camera, size_t subpixel_level,
        float sampling_step, pbvr::UnstructuredVolumeObject* volume );

    void createGT5D_full(
        Argument& param, kvs::Camera& camera, size_t subpixel_level,
        float sampling_step, pbvr::UnstructuredVolumeObject* volume );

    void createFromFile(
        Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step );

    void createFromFile(
        Argument& param, kvs::Camera& camera, size_t subpixel_level, float sampling_step, int st, int vl );

    pbvr::PointObject* getPointObject()
    {
        return object;
    }

    std::string getErrorMessage( const size_t maxMemory ) const;

    void setFinlterInfo( const FilterInfo& fi )
    {
        m_fi = &fi;
    }

private:
    pbvr::PointObject* sampling( Argument& param, kvs::Camera& camera, pbvr::VolumeObjectBase* volume, size_t subpixel_level, float sampling_step );

    pbvr::UnstructuredVolumeObject* read_avsucd( char* filename, const int componentId );
};

}

#endif
