#include "RangeEstimater.h"

VariableRange RangeEstimater::Estimation( const size_t step,
                                          const FilterInfo& fi,
                                          const TransferFunctionSynthesizer& tfs )
{
    VariableRange range;
    TransferFunctionSynthesizer s_min( tfs );
    TransferFunctionSynthesizer s_max( tfs );
    const QuantityMap& qmap = tfs.quantity_map();
    const TransferFunctionMap& fmap_c = tfs.color_transfunc_map();
    const TransferFunctionMap& fmap_o = tfs.opacity_transfunc_map();
    const TransfuncQuantityBindMap& bmap_c = tfs.color_transfunc_bind_map();
    const TransfuncQuantityBindMap& bmap_o = tfs.opacity_transfunc_bind_map();
    const FunctionMap& fmin = s_min.func_map();
    const FunctionMap& fmax = s_max.func_map();

    s_min.setValue( "X", fi.minObjectCoord.x() );
    s_min.setValue( "Y", fi.minObjectCoord.y() );
    s_min.setValue( "Z", fi.minObjectCoord.z() );
    s_max.setValue( "X", fi.maxObjectCoord.x() );
    s_max.setValue( "Y", fi.maxObjectCoord.y() );
    s_max.setValue( "Z", fi.maxObjectCoord.z() );

    for ( QuantityMap::const_iterator q = qmap.begin(); q != qmap.end(); q++ )
    {
        const std::string& nm = q->first;
        const size_t qi = q->second;
        s_min.setValue( nm, fi.ingredStep[step].ingred[qi].min );
        s_max.setValue( nm, fi.ingredStep[step].ingred[qi].max );
    }

    for ( FunctionMap::const_iterator i = fmin.begin(); i != fmin.end(); i++ )
    {
        const std::string& nm = i->first;
        const FuncParser::Function& f = i->second;
        s_min.setValue( nm, f.eval() );
    }

    for ( FunctionMap::const_iterator i = fmax.begin(); i != fmax.end(); i++ )
    {
        const std::string& nm = i->first;
        const FuncParser::Function& f = i->second;
        s_max.setValue( nm, f.eval() );
    }

    range.clear();
    for ( TransferFunctionMap::const_iterator f = fmap_c.begin(); f != fmap_c.end(); f++ )
    {
        const std::string& fnm = f->first;
        const std::string& qnm = bmap_c.find( fnm )->second;
        const std::string tag = fnm + "_var_c";

        range.setValue( tag, s_min.getValue( qnm ) );
        range.setValue( tag, s_max.getValue( qnm ) );
    }

    for ( TransferFunctionMap::const_iterator f = fmap_o.begin(); f != fmap_o.end(); f++ )
    {
        const std::string& fnm = f->first;
        const std::string& qnm = bmap_c.find( fnm )->second;
        const std::string tag = fnm + "_var_o";

        range.setValue( tag, s_min.getValue( qnm ) );
        range.setValue( tag, s_max.getValue( qnm ) );
    }

    return range;
}

