
#include "TransferFunctionSynthesizerCreator.h"
#include <stdio.h>
#include <string.h>

TransferFunctionSynthesizerCreator::TransferFunctionSynthesizerCreator()
{
    NamedTransferFunction tf;
    VolumeEquation ve_c, ve_o;
    tf.selection  = NamedTransferFunction::SelectTransferFunction;
    tf.Name       = "t1";
    tf.ColorVar   = "_t1_var_c";
    tf.OpacityVar = "_t1_var_o";
    ve_c.Name     = tf.ColorVar;
    ve_o.Name     = tf.OpacityVar;
    ve_c.Equation = "q1";
    ve_o.Equation = "q1";
    m_transfunc.push_back( tf );
    m_voleqn.push_back( ve_c );
    m_voleqn.push_back( ve_o );
    m_transfunc_synthesis = "t1";
}

void TransferFunctionSynthesizerCreator::setFilterInfo( const FilterInfo& fi )
{
    this->initQuantityMap( fi.numIngredients );
}

void TransferFunctionSynthesizerCreator::initQuantityMap( const int32_t numIngredients )
{
    m_qmap.clear();
    for ( int32_t i = 0; i < numIngredients; i++ )
    {
        std::stringstream ss;
        ss << "q" << ( i + 1 );

        const std::string name = ss.str();
        const size_t idx = i;

        m_qmap[name] = idx;
    }
}

void TransferFunctionSynthesizerCreator::setRange( const std::string name, const float min_value, const float max_value )
{
    setColorRange( name, min_value, max_value );
    setOpacityRange( name, min_value, max_value );
}

void TransferFunctionSynthesizerCreator::setColorRange( const std::string& name, const float min_value, const float max_value )
{
    for ( size_t i = 0; i < m_transfunc.size(); i++ )
    {
        if ( m_transfunc[i].Name == name )
        {
            m_transfunc[i].ColorVarMin = min_value;
            m_transfunc[i].ColorVarMax = max_value;
        }
    }
}

void TransferFunctionSynthesizerCreator::setOpacityRange( const std::string& name, const float min_value, const float max_value )
{
    for ( size_t i = 0; i < m_transfunc.size(); i++ )
    {
        if ( m_transfunc[i].Name == name )
        {
            m_transfunc[i].OpacityVarMin = min_value;
            m_transfunc[i].OpacityVarMax = max_value;
        }
    }
}

void TransferFunctionSynthesizerCreator::setParamInfo( const ParamInfo& pa )
{
    set_param_info( pa );
}

void TransferFunctionSynthesizerCreator::setProtocol( const jpv::ParticleTransferClientMessage& clntMes )
{
    set_protocol( clntMes );
}

void TransferFunctionSynthesizerCreator::setAsisTransferFunction( const pbvr::TransferFunction tf )
{
}

void TransferFunctionSynthesizerCreator::setTransferFunction( const std::string& name, const pbvr::TransferFunction& tf )
{
    for ( size_t i = 0; i < m_transfunc.size(); i++ )
    {
        if ( m_transfunc[i].Name == name )
        {
            NamedTransferFunction ntf( tf );
            ntf.selection = NamedTransferFunction::SelectTransferFunction;
            ntf.Name       = name;
            ntf.ColorVar   = "_" + name + "_var_c";
            ntf.OpacityVar = "_" + name + "_var_o";
            m_transfunc[i] = ntf;
        }
    }
}

TransferFunctionSynthesizerCreator::~TransferFunctionSynthesizerCreator()
{
}

TransferFunctionSynthesizer* TransferFunctionSynthesizerCreator::create()
{
    m_synthesizer = new TransferFunctionSynthesizer();
    m_synthesizer->clear();

    this->assign();
    m_synthesizer->setQuantityMap( m_qmap );

    m_synthesizer->optimize();
    m_synthesizer->create();

    return ( m_synthesizer );
}

void TransferFunctionSynthesizerCreator::assign()
{
    TransferFunctionMap      tfm_c, tfm_o;
    TransfuncQuantityBindMap tqm_c, tqm_o;
    FunctionStringMap        vfm;

    const std::string tf_synth = m_transfunc_synthesis;

    for ( size_t i = 0; i < m_transfunc.size(); i++ )
    {
        const NamedTransferFunction::Selection s = m_transfunc[i].selection;
        const std::string& name = m_transfunc[i].Name;
        const std::string& var_c = m_transfunc[i].ColorVar;
        const std::string& var_o = m_transfunc[i].OpacityVar;
        const std::string& R = m_transfunc[i].EquationR;
        const std::string& G = m_transfunc[i].EquationG;
        const std::string& B = m_transfunc[i].EquationB;
        const std::string& A = m_transfunc[i].EquationA;
        const int32_t resolution = m_transfunc[i].Resolution;
        const float min_c = m_transfunc[i].ColorVarMin;
        const float max_c = m_transfunc[i].ColorVarMax;
        const float min_o = m_transfunc[i].OpacityVarMin;
        const float max_o = m_transfunc[i].OpacityVarMax;

        if ( s == NamedTransferFunction::SelectExtendTransferFunction )
        {
            tfm_c[name] = ExtendedTransferFunction( R, G, B, A, resolution, min_c, max_c );
            tfm_o[name] = ExtendedTransferFunction( R, G, B, A, resolution, min_o, max_o );
        }
        else if ( s == NamedTransferFunction::SelectTransferFunction )
        {
            tfm_c[name] = static_cast<pbvr::TransferFunction>( m_transfunc[i] );
            tfm_o[name] = static_cast<pbvr::TransferFunction>( m_transfunc[i] );
            tfm_c[name].setRange( min_c, max_c );
            tfm_o[name].setRange( min_o, max_o );
        }

        tqm_c[name] = var_c;
        tqm_o[name] = var_o;
    }

    for ( size_t i = 0; i < m_voleqn.size(); i++ )
    {
        const std::string& name  = m_voleqn[i].Name;
        const std::string& value = m_voleqn[i].Equation;
        vfm[name] = value;
    }

    m_synthesizer->setColorTransferFunctionMap( tfm_c );
    m_synthesizer->setOpacityTransferFunctionMap( tfm_o );
    m_synthesizer->setColorTransfuncQuantityBindMap( tqm_c );
    m_synthesizer->setOpacityTransfuncQuantityBindMap( tqm_o );
    m_synthesizer->setVolumeFunctionStringMap( vfm );
    m_synthesizer->setSynthFunctionString( tf_synth );
}

void TransferFunctionSynthesizerCreator::set_protocol( const jpv::ParticleTransferClientMessage& clntMes )
{
    m_transfunc.clear();
    m_voleqn.clear();

    m_transfunc_synthesis = clntMes.transferFunctionSynthesis;

    for ( size_t i = 0; i < clntMes.transfunc.size(); i++ )
    {
        NamedTransferFunction tf;

        tf.Name          = clntMes.transfunc[i].Name;
        tf.ColorVar      = clntMes.transfunc[i].ColorVar;
        tf.ColorVarMin   = clntMes.transfunc[i].ColorVarMin;
        tf.ColorVarMax   = clntMes.transfunc[i].ColorVarMax;
        tf.OpacityVar    = clntMes.transfunc[i].OpacityVar;
        tf.OpacityVarMin = clntMes.transfunc[i].OpacityVarMin;
        tf.OpacityVarMax = clntMes.transfunc[i].OpacityVarMax;
        tf.Resolution    = clntMes.transfunc[i].Resolution;
        tf.EquationR     = clntMes.transfunc[i].EquationR;
        tf.EquationG     = clntMes.transfunc[i].EquationG;
        tf.EquationB     = clntMes.transfunc[i].EquationB;
        tf.EquationA     = clntMes.transfunc[i].EquationA;
        tf.setColorMap( clntMes.transfunc[i].colorMap() );
        tf.setOpacityMap( clntMes.transfunc[i].opacityMap() );

        if ( clntMes.transfunc[i].selection == NamedTransferFunctionParameter::SelectExtendTransferFunction )
        {
            tf.selection = NamedTransferFunction::SelectExtendTransferFunction;
        }
        else if ( clntMes.transfunc[i].selection == NamedTransferFunctionParameter::SelectTransferFunction )
        {
            tf.selection = NamedTransferFunction::SelectTransferFunction;
        }

        m_transfunc.push_back( tf );
    }

    for ( size_t i = 0; i < clntMes.voleqn.size(); i++ )
    {
        VolumeEquation ve;
        ve.Name     = clntMes.voleqn[i].Name;
        ve.Equation = clntMes.voleqn[i].Equation;
        m_voleqn.push_back( ve );
    }
}

void TransferFunctionSynthesizerCreator::set_param_info( const ParamInfo& pa )
{
    m_transfunc.clear();
    m_voleqn.clear();

    ParamInfo   pa1 = const_cast<ParamInfo&>( pa );

    // value check
    if ( !pa1.getState( "TF_RESOLUTION" ) || !pa1.getState( "TF_SYNTH" ) )
    {
        return;
    }

    int32_t resolution = static_cast<int32_t>( pa1.getInt( "TF_RESOLUTION" ) );
    m_transfunc_synthesis =   pa1.getString( "TF_SYNTH" );
    // printf(" [debug] transfunc_synth: %s\n", m_transfunc_synthesis.c_str());
    // printf(" [debug] resolution = %d\n", resolution);

    for ( size_t i = 0; i < TF_COUNT; i++ )
    {
        NamedTransferFunction tf;
        VolumeEquation        ve_c, ve_o;

        std::stringstream tss;
        tss << "TF_NAME" << i + 1 << "_";
        const std::string tag_base = tss.str();

        // value check
        if ( !pa1.getState( tag_base + "MIN_C" ) ||
                !pa1.getState( tag_base + "MAX_C" ) ||
                !pa1.getState( tag_base + "MIN_O" ) ||
                !pa1.getState( tag_base + "MAX_O" ) ||
                !pa1.getState( tag_base + "VAR_C" ) ||
                !pa1.getState( tag_base + "VAR_O" ) ||
                !pa1.getState( tag_base + "TABLE_C" ) ||
                !pa1.getState( tag_base + "TABLE_O" ) ) continue;

        std::stringstream ss;
        ss << "t" << i + 1;
        const std::string name = ss.str();
        tf.create( ( size_t )resolution );
        tf.Name = name;
        tf.Resolution = resolution;
        tf.ColorVarMin = pa1.getFloat( tag_base + "MIN_C" );
        tf.ColorVarMax = pa1.getFloat( tag_base + "MAX_C" );
        tf.OpacityVarMin = pa1.getFloat( tag_base + "MIN_O" );
        tf.OpacityVarMax = pa1.getFloat( tag_base + "MAX_O" );
        ve_c.Equation = pa1.getString( tag_base + "VAR_C" );
        ve_o.Equation = pa1.getString( tag_base + "VAR_O" );

        kvs::ColorMap::Table colorTable( resolution * 3 );
        std::string s_color = pa1.getString( tag_base + "TABLE_C" );
        char* cp_color = ( char* )s_color.c_str();
        for ( int32_t j = 0; j < resolution * 3; j++ )
        {
            int val;
            sscanf( cp_color, "%d,", &val );
            colorTable[j] = val;
            cp_color = strstr( cp_color, "," ) + 1;
        }

        kvs::OpacityMap::Table opacityTable( resolution );
        std::string s_opacity = pa1.getString( tag_base + "TABLE_O" );
        char* cp_opacity = ( char* )s_opacity.c_str();
        for ( int32_t j = 0; j < resolution; j++ )
        {
            float v_opacity;
            sscanf( cp_opacity, "%f,", &v_opacity );
            opacityTable[j] = v_opacity;
            cp_opacity = strstr( cp_opacity, "," ) + 1;
        }

        kvs::ColorMap colorMap( colorTable );
        kvs::OpacityMap opacityMap( opacityTable );
        colorMap.setResolution( ( size_t )resolution );
        colorMap.setRange( tf.ColorVarMin, tf.ColorVarMax );
        opacityMap.setResolution( resolution );
        opacityMap.setRange( tf.OpacityVarMin, tf.OpacityVarMax );
        tf.setColorMap( colorMap );
        tf.setOpacityMap( opacityMap );

        std::stringstream fss;
        fss << "_F" << i + 1;
        const std::string fname = fss.str();
        const std::string cname = fname + "_VAR_C";
        const std::string oname = fname + "_VAR_O";
        tf.ColorVar   = cname;
        tf.OpacityVar = oname;
        ve_c.Name = cname;
        ve_o.Name = oname;

        tf.selection  = NamedTransferFunction::SelectTransferFunction;

        m_transfunc.push_back( tf );
        m_voleqn.push_back( ve_c );
        m_voleqn.push_back( ve_o );

        // debug
        // printf(" [debug] trans func: %s\n", tf.Name.c_str());
        // printf(" [debug]   resolution = %d\n", tf.Resolution);
        // printf(" [debug]   ColorVarMin = %f\n", tf.ColorVarMin);
        // printf(" [debug]   ColorVarMax = %f\n", tf.ColorVarMax);
        // printf(" [debug]   OpacityVarMin = %f\n", tf.OpacityVarMin);
        // printf(" [debug]   OpacityVarMax = %f\n", tf.OpacityVarMax);
        // printf(" [debug]   ColorVar = %s\n", tf.ColorVar.c_str());
        // printf(" [debug]   OpacityVar = %s\n", tf.OpacityVar.c_str());
        // printf(" [debug]   co = ");
        // for(int32_t j=0; j<resolution; j++){
        //   printf("(%d,%d,%d,%f),",
        //          colorTable[3*j+0], colorTable[3*j+1], colorTable[3*j+2], opacityTable[j]);
        // }
        // printf("\n");
    }
}

